/**
 * hpgv_gl.c
 *
 * Copyright (c) 2008 Hongfeng Yu
 *
 * Contact:
 * Hongfeng Yu
 * hfstudio@gmail.com
 * 
 * 
 * All rights reserved.  May not be used, modified, or copied 
 * without permission.
 *
 */

#include "hpgv_error.h"
#include "hpgv_gl.h"
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>

/**
 * hpgv_typesize
 *
 */
int 
hpgv_typesize(int type) {
    switch (type) {
        case HPGV_BYTE : return 1;
        case HPGV_UNSIGNED_BYTE : return 1;
        case HPGV_SHORT : return 2;
        case HPGV_UNSIGNED_SHORT : return 2;
        case HPGV_INT : return 4;
        case HPGV_UNSIGNED_INT : return 4;
        case HPGV_FLOAT : return 4;
        case HPGV_DOUBLE : return 8;
    }
    
    return 0;
}


/**
 * hpgv_formatsize
 *
 */
int
hpgv_formatsize(int format)
{
    switch (format) {        
        case HPGV_RGB : return 3;
        case HPGV_RGBA : return 4;        
    }    
    return 0;    
}



/**
 * hpgv_vis_saveppm  
 *
 */
int  
hpgv_vis_saveppm(int width, int height, int format, int type, 
                 const void *pixels, char *filename)
{
    uint64_t i;
    unsigned char r, g, b;
    int t, x, y;
    float *framebuff;
    double *framebufd;
    unsigned char *framebufc;
    
    if (format != HPGV_RGB && format != HPGV_RGBA) {
        fprintf(stderr, "Unsupported pixel format.\n");
        return HPGV_ERROR;
    }
    
    if (type != HPGV_FLOAT && type != HPGV_DOUBLE && type != HPGV_UNSIGNED_BYTE) 
    {
        fprintf(stderr, "Unsupported pixel type.\n");
        return HPGV_ERROR;
    }
        
    FILE *fp;
    
    fp = fopen(filename, "w" );
    if (!fp) {
        fprintf(stderr, "Can not open file %s.\n", filename);
        return HPGV_ERROR;
    }
    
    fprintf(fp,"P6\n");    
    fprintf(fp,"%i %i\n", width, height);
    fprintf(fp,"255\n");
    fclose(fp);
    
    /* reopen in binary append mode */
    fp = fopen( filename, "ab" );  
    if (!fp) {
        fprintf(stderr, "Can not open file %s.\n", filename);
        return HPGV_ERROR;
    }
        
    t = hpgv_formatsize(format);
    
    if (type == HPGV_FLOAT) {
        framebuff = (float *)pixels;
    } else if (type == HPGV_DOUBLE) {
        framebufd = (double *)pixels;
    } else if (type == HPGV_UNSIGNED_BYTE) {
        framebufc = (unsigned char *) pixels;
    }
    
    for (y = height - 1; y >= 0; y--) {
        for (x = 0; x < width; x++) {
            i = x+ y * width;
                    
            if (type == HPGV_FLOAT) {                
                r = (unsigned char)(framebuff[i * t] * 255.0);
                g = (unsigned char)(framebuff[i * t + 1] * 255.0);
                b = (unsigned char)(framebuff[i * t + 2] * 255.0);
            } else if (type == HPGV_DOUBLE) {
                r = (unsigned char)(framebufd[i * t] * 255.0);
                g = (unsigned char)(framebufd[i * t + 1] * 255.0);
                b = (unsigned char)(framebufd[i * t + 2] * 255.0);                
            }else if (type == HPGV_UNSIGNED_BYTE) {
                r = framebufc[i * t];
                g = framebufc[i * t + 1];
                b = framebufc[i * t + 2];
            }
            
            fputc(r, fp);
            fputc(g, fp);
            fputc(b, fp);
        }
    }
    
    fclose(fp);
    
    return HPGV_SUCCESS;
}
