/**
 * hpgv_utilmath.c
 *
 * Copyright (c) 2008 Hongfeng Yu
 *
 * Contact:
 * Hongfeng Yu
 * hfstudio@gmail.com
 * 
 * 
 * All rights reserved.  May not be used, modified, or copied 
 * without permission.
 *
 */

#include "hpgv_utilmath.h"

/**
 * mat_time_mat
 *
 */
void 
mat_time_mat(const double a[16], const double b[16], double c[16])
{
    int i, j, m;
    for (i = 0; i < 4; i++){
        for (j = 0; j < 4; j++){
            double value = 0;
            for (m = 0; m < 4; m++)
                value += a[m + i * 4] * b[j + m * 4];
            c[j+i*4] = value; 
        }
    }
}


/**
 * inverse_mat
 *
 */
int
inverse_mat(const double a[16], double b[16])
{
    #define a(i,j) (a[(i-1)*4 + (j-1)])
    #define b(i,j) (b[(i-1)*4 + (j-1)])

    double detA =  
            a(1,1)*a(2,2)*a(3,3)*a(4,4) + a(1,1)*a(2,3)*a(3,4)*a(4,2)
          + a(1,1)*a(2,4)*a(3,2)*a(4,3)
          + a(1,2)*a(2,1)*a(3,4)*a(4,3) + a(1,2)*a(2,3)*a(3,1)*a(4,4)
          + a(1,2)*a(2,4)*a(3,3)*a(4,1)
          + a(1,3)*a(2,1)*a(3,2)*a(4,4) + a(1,3)*a(2,2)*a(3,4)*a(4,1)
          + a(1,3)*a(2,4)*a(3,1)*a(4,2)
          + a(1,4)*a(2,1)*a(3,3)*a(4,2) + a(1,4)*a(2,2)*a(3,1)*a(4,3) 
          + a(1,4)*a(2,3)*a(3,2)*a(4,1)
          - a(1,1)*a(2,2)*a(3,4)*a(4,3) - a(1,1)*a(2,3)*a(3,2)*a(4,4)
          - a(1,1)*a(2,4)*a(3,3)*a(4,2)
          - a(1,2)*a(2,1)*a(3,3)*a(4,4) - a(1,2)*a(2,3)*a(3,4)*a(4,1)
          - a(1,2)*a(2,4)*a(3,1)*a(4,3)
          - a(1,3)*a(2,1)*a(3,4)*a(4,2) - a(1,3)*a(2,2)*a(3,1)*a(4,4)
          - a(1,3)*a(2,4)*a(3,2)*a(4,1)
          - a(1,4)*a(2,1)*a(3,2)*a(4,3) - a(1,4)*a(2,2)*a(3,3)*a(4,1)
          - a(1,4)*a(2,3)*a(3,1)*a(4,2);

    if (detA == 0)
        return HPGV_ERROR;

    b(1,1) = 
     (a(2,2)*a(3,3)*a(4,4) + a(2,3)*a(3,4)*a(4,2) + a(2,4)*a(3,2)*a(4,3)
    - a(2,2)*a(3,4)*a(4,3) - a(2,3)*a(3,2)*a(4,4) -a(2,4)*a(3,3)*a(4,2))/detA;
    
    b(1,2) = 
     (a(1,2)*a(3,4)*a(4,3) + a(1,3)*a(3,2)*a(4,4) + a(1,4)*a(3,3)*a(4,2)
    - a(1,2)*a(3,3)*a(4,4) - a(1,3)*a(3,4)*a(4,2) - a(1,4)*a(3,2)*a(4,3))/detA;
    
    b(1,3) = 
     (a(1,2)*a(2,3)*a(4,4) + a(1,3)*a(2,4)*a(4,2) + a(1,4)*a(2,2)*a(4,3)
    - a(1,2)*a(2,4)*a(4,3) - a(1,3)*a(2,2)*a(4,4) - a(1,4)*a(2,3)*a(4,2))/detA;
    
    b(1,4) = 
     (a(1,2)*a(2,4)*a(3,3) + a(1,3)*a(2,2)*a(3,4) + a(1,4)*a(2,3)*a(3,2) 
    - a(1,2)*a(2,3)*a(3,4) - a(1,3)*a(2,4)*a(3,2) - a(1,4)*a(2,2)*a(3,3))/detA;
    
    b(2,1) = 
     (a(2,1)*a(3,4)*a(4,3) + a(2,3)*a(3,1)*a(4,4) + a(2,4)*a(3,3)*a(4,1) 
    - a(2,1)*a(3,3)*a(4,4) - a(2,3)*a(3,4)*a(4,1) - a(2,4)*a(3,1)*a(4,3))/detA;
    
    b(2,2) = 
     (a(1,1)*a(3,3)*a(4,4) + a(1,3)*a(3,4)*a(4,1) + a(1,4)*a(3,1)*a(4,3) 
    - a(1,1)*a(3,4)*a(4,3) - a(1,3)*a(3,1)*a(4,4) - a(1,4)*a(3,3)*a(4,1))/detA;
    
    b(2,3) = 
     (a(1,1)*a(2,4)*a(4,3) + a(1,3)*a(2,1)*a(4,4) + a(1,4)*a(2,3)*a(4,1) 
    - a(1,1)*a(2,3)*a(4,4) - a(1,3)*a(2,4)*a(4,1) - a(1,4)*a(2,1)*a(4,3))/detA;
    
    b(2,4) = 
     (a(1,1)*a(2,3)*a(3,4) + a(1,3)*a(2,4)*a(3,1) + a(1,4)*a(2,1)*a(3,3) 
    - a(1,1)*a(2,4)*a(3,3) - a(1,3)*a(2,1)*a(3,4) - a(1,4)*a(2,3)*a(3,1))/detA;
    
    b(3,1) = 
     (a(2,1)*a(3,2)*a(4,4) + a(2,2)*a(3,4)*a(4,1) + a(2,4)*a(3,1)*a(4,2) 
    - a(2,1)*a(3,4)*a(4,2) - a(2,2)*a(3,1)*a(4,4) - a(2,4)*a(3,2)*a(4,1))/detA;
    
    b(3,2) = 
     (a(1,1)*a(3,4)*a(4,2) + a(1,2)*a(3,1)*a(4,4) + a(1,4)*a(3,2)*a(4,1) 
    - a(1,1)*a(3,2)*a(4,4) - a(1,2)*a(3,4)*a(4,1) - a(1,4)*a(3,1)*a(4,2))/detA;
    
    b(3,3) = 
     (a(1,1)*a(2,2)*a(4,4) + a(1,2)*a(2,4)*a(4,1) + a(1,4)*a(2,1)*a(4,2) 
    - a(1,1)*a(2,4)*a(4,2) - a(1,2)*a(2,1)*a(4,4) - a(1,4)*a(2,2)*a(4,1))/detA;
    
    b(3,4) = 
     (a(1,1)*a(2,4)*a(3,2) + a(1,2)*a(2,1)*a(3,4) + a(1,4)*a(2,2)*a(3,1) 
    - a(1,1)*a(2,2)*a(3,4) - a(1,2)*a(2,4)*a(3,1) - a(1,4)*a(2,1)*a(3,2))/detA;
    
    b(4,1) = 
     (a(2,1)*a(3,3)*a(4,2) + a(2,2)*a(3,1)*a(4,3) + a(2,3)*a(3,2)*a(4,1) 
    - a(2,1)*a(3,2)*a(4,3) - a(2,2)*a(3,3)*a(4,1) - a(2,3)*a(3,1)*a(4,2))/detA;
    
    b(4,2) = 
     (a(1,1)*a(3,2)*a(4,3) + a(1,2)*a(3,3)*a(4,1) + a(1,3)*a(3,1)*a(4,2)
    - a(1,1)*a(3,3)*a(4,2) - a(1,2)*a(3,1)*a(4,3) - a(1,3)*a(3,2)*a(4,1))/detA;
    
    b(4,3) = 
     (a(1,1)*a(2,3)*a(4,2) + a(1,2)*a(2,1)*a(4,3) + a(1,3)*a(2,2)*a(4,1) 
    - a(1,1)*a(2,2)*a(4,3) - a(1,2)*a(2,3)*a(4,1) - a(1,3)*a(2,1)*a(4,2))/detA;
    
    b(4,4) = 
     (a(1,1)*a(2,2)*a(3,3) + a(1,2)*a(2,3)*a(3,1) + a(1,3)*a(2,1)*a(3,2) 
    - a(1,1)*a(2,3)*a(3,2) - a(1,2)*a(2,1)*a(3,3) - a(1,3)*a(2,2)*a(3,1))/detA;

    return HPGV_SUCCESS;
}

/**
 * normalize 
 *
 */
int
normalize(point_3d_t *p3d)
{
    double len = SQR(p3d->x3d) + SQR(p3d->y3d) + SQR(p3d->z3d);
    len = sqrt(len);

    if (len != 0){
        p3d->x3d /= len;
        p3d->y3d /= len;
        p3d->z3d /= len;
        return HPGV_SUCCESS;    
    }
    return HPGV_ERROR;
}
